#CSE 378 BIOS
#Version 2.0 - Rewritten by Adrian for Lab 4
#4/29/2007

#t1 for command code
#t2 for location in memory to store
#t3 for data to store
#t8 for temporary for checking vals
#t9 for bootloader base
#t6 is flush counter
#s0 stores address of LED controller
#s1 stores current LED status
	  					
#serial i/o:
#byte 0: 1 if data available to read
#byte 1: data at head of read queue
#byte 2: send fifo full

#store to 3 advances read pointer
#store to 4 appends to write queue

addi $t0, $0, 1
lui $t1, 0x8000
ori $t1, $t1, 0x0080
lwl $t0, 0($t1)
bne $t0, $0, secondProc
nop

init:
	lui		$t3, 0x7fff		#Put a flag so we know when to jump out
	ori		$t3, $t3, 0x0000
	ori		$t4, $0, 1		#Non-zero flag
	sw		$t4, 0($t3)

	lui		$s0, 0x8000		#address of IO Devices
	addi	$t0, $0, 1		#turn on LED 1 only
	sw		$t0, 0x40($s0)	#store to LEDS

main:
	jal		GetByte			#jump to byte read routine
	nop
	addi	$t0, $0, 0		#set Code to 0
	beq		$v0, $t0, StoreWord
	addi	$t0, $t0, 1
	beq		$v0, $t0, SetAddress
	addi	$t0, $t0, 1
	beq		$v0, $t0, Done
	addi	$t0, $t0, 1
	beq		$v0, $t0, StoreByte
	nop
	j		Error
	nop

GetByte:
	lbu		$t0, 0($s0)		#get data available bit
	nop
	beq		$t0, $0, GetByte #loop if no data available
	nop
	lbu		$v0, 1($s0)		#load byte
	jr		$ra				#return
	sb		$0, 3($s0)		#advance read pointer			
	
GetWord:
	add		$t2, $ra, $0	#save RA
	addi	$t1, $0, 4		#setup counter
	add		$v1, $0, $0		#clear dest register
GetWord_Loop:
	jal		GetByte
	addi	$t1, $t1, -1	#decrement counter
	sll		$v0, $v0, 24	#shift new byte into byte 3 position
	srl		$v1, $v1, 8		#shift previous read bytes down one byte
	bne		$t1, $0, GetWord_Loop
	or		$v1, $v0, $v1	#or in new data
	jr		$t2				#return		   
	add		$v0, $v1, $0	#move result into v0


SetAddress:
	jal		GetWord			#read in address
	nop		
	j		main
	add		$k0, $v0, $0	#set address

StoreWord:
	jal		GetWord			#read in word
	nop
	sw		$v0, 0($k0)		#store to current write pointer
	j		main
	addi	$k0, $k0, 4		#increment write pointer
	
StoreByte:
	jal		GetWord			#read in word (only use lowest byte)
	nop
	sw		$v0, 0($k0)		#store to current write pointer
	j		main
	addi	$k0, $k0, 1		#increment write pointer

Done:					
	lui		$t0, 0x8000
	ori		$t0, $t0, 0x0040
	sw		$t0, 0($t0)
	lui		$t0, 0x7fff  	#setup registers for cache flush loop
	addi	$t1, $0, 256		  

flushloop:
	lw		$1, 0($t0)		#load a word
	addiu	$t0, $t0, 4		#advance pointer to next word
	bne		$t1, $0, flushloop	#see if we've read enough words
	addiu	$t1, $t1, -1	#decrement counter
	
endsecondProc:
	lui $t1, 0x7fff
	ori $t1, $t1, 0x0000
	sw $0, 0($t1)
												
finalizeAndJump:
	lui		$sp, 0x7fff		#setup stack pointer
	ori		$sp, $sp, 0xfffc
	lui		$ra, 0x0040		#setup return address
	ori		$ra, $ra, 0
	jr		$ra				#jump into user code

Error:
	ori		$t0, $t0, 0x8	#led 3 on
	sw		$t0, 0x40($s0)	#set LEDS
ErrorLoop:
	j		ErrorLoop
	nop
	
secondProc:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	lui 	$t1, 0x7fff
	ori 	$t1, $t1, 0x0000	

sploop:
	lw $t0, 0($t1)
	bne $t0, $0, sploop
	nop
	j finalizeAndJump 
