`timescale 1ns/1ps

module IOWedge
	(
	input 	[31:0] 	DataFromProcessor,
	output 	[31:0] 	DataToProcessor,
	
	input 	[31:0] 	DataFromIO,
	output 	[31:0] 	DataToIO,
	
	input	[31:0]	DataFromCache,
	output	[31:0]	DataToCache,
	
	input 	[31:0] 	AddressFromProcessor,
	output 	[31:0] 	AddressToIO, 
	output	[31:0]	AddressToCache,
	
	input 			Read,
	input 			Write,
	input 	[1:0]	Size, 
	input			SignedLoad,
	
	output 			IORead,
	output 			IOWrite,
	output 	[1:0] 	IOSize,
	
	output 			CacheRead,
	output 			CacheWrite,
	output 	[1:0]	CacheSize,
	
	input			IOStall,
	input			CacheStall,
	output			Stall
	);			  
	
	wire IOSelect;
	wire [31:0] DataSelect;
	wire [15:0] selectedHalfword;
	wire [7:0] selectedByte;
	
	assign	IOSelect = AddressFromProcessor[31];
	
	assign	Stall = IOSelect ? IOStall : CacheStall;
	
	assign	CacheRead 	= IOSelect ? 1'b0 : Read;
	assign	CacheWrite	= IOSelect ? 1'b0 : Write;
	assign	CacheSize 	= Size;
	
	assign	IORead		= IOSelect ? Read : 1'b0;
	assign	IOWrite		= IOSelect ? Write : 1'b0;
	assign 	IOSize		= Size;
	
	
	assign	AddressToIO	= AddressFromProcessor;
	assign	AddressToCache	= AddressFromProcessor;
	
	assign	DataToIO	= DataFromProcessor;
	assign	DataToCache	= DataFromProcessor;
	
	assign	DataSelect	 = IOSelect ? DataFromIO : DataFromCache;
	
	assign selectedByte = (AddressFromProcessor[1:0] == 2'b00) ? DataSelect[7:0] : (AddressFromProcessor[1:0] == 2'b01) ? DataSelect[15:8] :
	(AddressFromProcessor[1:0] == 2'b10) ? DataSelect[23:16] : DataSelect[31:24];
	
	assign selectedHalfword = (~AddressFromProcessor[1]) ? DataSelect[15:0] : DataSelect[31:15];
	
	localparam BYTE =2'b01;
	localparam HALF =2'b10;
	localparam WORD =2'b11;
	
	assign DataToProcessor = 
	(Size == WORD) ? DataSelect : 
	(Size == BYTE) ? ((SignedLoad) ? {{24{selectedByte[7]}}, selectedByte[7:0]} : {24'b0, selectedByte[7:0]}) : 
	/*Size==HALF ?*/ ((SignedLoad) ? {{16{selectedHalfword[15]}}, selectedHalfword[15:0]} : {16'b0, selectedHalfword[15:0]});
	
endmodule